﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;


namespace JayJayWon
{
    public class CustomUIElement
    {
        protected UIDynamic _spacerLeft { get; set; }
        protected UIDynamic _spacerRight { get; set; }
        protected UIDynamic _spacerActive { get; set; }

        protected float spacerHeight;
        public float height { get { return spacerHeight; } }
        protected HashSet<UIDynamicPopup> uidPopuplist;


        public CustomUIElement(float height)
        {
            spacerHeight = height;
            uidPopuplist = new HashSet<UIDynamicPopup>();
        }

        public void CloseAllPopups(UIDynamicPopup exception)
        {
            foreach (UIDynamicPopup popup in uidPopuplist)
            {
                if (popup != exception) popup.popup.visible = false;
            }
        }

        public virtual void ClearUI()
        {
            if (_spacerLeft != null) Globals.mvrScript.RemoveSpacer(_spacerLeft);
            if (_spacerRight != null) Globals.mvrScript.RemoveSpacer(_spacerRight);
            _spacerActive = null;
            uidPopuplist.Clear();
        }

        protected UIDynamicPopup InstantiatePopup(JSONStorableStringChooser jssc, bool scrollable)
        {
            Transform originalTransform = Globals.mvrScript.manager.configurableScrollablePopupPrefab.transform;
            if (!scrollable) originalTransform = Globals.mvrScript.manager.configurablePopupPrefab.transform;

            Transform popup = UnityEngine.Object.Instantiate(originalTransform, _spacerActive.gameObject.transform, false);
            UIDynamicPopup selectTargetPopup = popup.GetComponent<UIDynamicPopup>();

            selectTargetPopup.popup.showSlider = false;
            popup.SetParent(_spacerActive.gameObject.transform, false);
            jssc.popup = selectTargetPopup.popup;
            return (selectTargetPopup);
        }
        protected UIDynamicSlider InstantiateSlider(JSONStorableFloat jsf)
        {
            Transform originalTransform = Globals.mvrScript.manager.configurableSliderPrefab.transform;

            Transform slider = UnityEngine.Object.Instantiate(originalTransform, _spacerActive.gameObject.transform, false);
            UIDynamicSlider newSlider = slider.GetComponent<UIDynamicSlider>();

            slider.SetParent(_spacerActive.gameObject.transform, false);
            jsf.slider = newSlider.slider;
            return (newSlider);
        }

        private UIDynamicSlider CreateSlider(JSONStorableFloat jsf, float width, float height, float xOffsetPosition, float yOffsetPostion = 0f)
        {
            RectTransform spacerRT = _spacerActive.GetComponent<RectTransform>();

            spacerRT.anchorMin = new Vector2(0f, 1f);
            spacerRT.anchorMax = new Vector2(0f, 1f);
            spacerRT.pivot = new Vector2(0f, 1f);

            UIDynamicSlider newSlider = InstantiateSlider(jsf);
            
            RectTransform childRect = newSlider.transform.GetComponent<RectTransform>();
            childRect.anchorMin = new Vector2(0f, 1f);
            childRect.anchorMax = new Vector2(0f, 1f);
            childRect.pivot = new Vector2(0f, 1f);

            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width);
            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
            childRect.transform.localPosition = new Vector3(xOffsetPosition, yOffsetPostion, 0f);

            return (newSlider);
        }

        // This prevents the popup panel from being rendered behind other items. Needs to be set at the end of the Frame to override the VAM default behaviour.
        protected IEnumerator ReparentPopup(UIDynamicPopup popupUID, bool forceRightParentPopup)
        {
            yield return new WaitForEndOfFrame();

            if (popupUID.popup.visible)
            {
                if (forceRightParentPopup) popupUID.popup.popupPanel.transform.SetParent(_spacerRight.gameObject.transform.parent);
                else popupUID.popup.popupPanel.transform.SetParent(_spacerActive.gameObject.transform.parent);
            }
        }

        public UIDynamicPopup AddPopup(JSONStorableStringChooser jssc, UIPopup.OnOpenPopup callbackMethod, float width, float height, float xOffsetPosition, float yOffsetPosition = 0, float panelHeight = 0f, float popupRowHeight=50f,string labelOverride = null, float labelProportion = 0.4f)
        {
            UIDynamicPopup newPopup;
            if (panelHeight == 0f) newPopup = UI.CreatePopup(_spacerActive.gameObject, jssc, width, height, labelOverride, 55, xOffsetPosition, yOffsetPosition, anchorPreset: AnchorPresets.topLeft, popupRowHeight: popupRowHeight, labelProportion: labelProportion);
            else newPopup = UI.CreateScrollablePopup(_spacerActive.gameObject, jssc, width, height, labelOverride, 55, xOffsetPosition, yOffsetPosition, panelHeight, false, true, AnchorPresets.topLeft, labelProportion, popupRowHeight);

            if (callbackMethod!=null) newPopup.popup.onOpenPopupHandlers += callbackMethod;

            newPopup.popup.onOpenPopupHandlers += () =>
            {
                //                UIAPluginUI.CloseAllPopups(newPopup);
                Globals.mvrScript.StartCoroutine(ReparentPopup(newPopup, false));
            };

            newPopup.popup.selectColor = Const.UIhighlightColor;

            uidPopuplist.Add(newPopup);

            return newPopup;
        }

        public UIDynamicToggle AddToggle (JSONStorableBool jsb, float width, float height, float xOffsetPosition, float yOffsetPosition, string labelOverride=null, float checkboxSize = 50f)
        {
            UIDynamicToggle newToggle= UI.CreateToggle(_spacerActive.gameObject,jsb,width,height, labelOverride,55,xOffsetPosition,yOffsetPosition,AnchorPresets.topLeft, checkboxSize);
            return newToggle;
        }
        public UIDynamicTextField AddTextBox(JSONStorableString jss, float width, float height, float xOffsetPosition, float yOffsetPosition)
        {

            UIDynamicTextField newTextField = UI.CreateTextBox(_spacerActive.gameObject, jss, width, height, 55, xOffsetPosition, yOffsetPosition, AnchorPresets.topLeft);
            return newTextField;
        }
        public UIDynamicTextField AddTextBox(string text, float width, float height, float xOffsetPosition, float yOffsetPosition)
        {
            JSONStorableString jss = new JSONStorableString("textField", text);
            UIDynamicTextField newTextField = UI.CreateTextBox(_spacerActive.gameObject, jss, width, height, 55, xOffsetPosition, yOffsetPosition, AnchorPresets.topLeft);
            return newTextField;
        }



        public UIDynamicButton AddButton(float width, float height, float xOffsetPosition, float yOffsetPosition, string label, UnityEngine.Events.UnityAction callback)
        {
            UIDynamicButton newButton = UI.CreateUIButton(_spacerActive.gameObject,width,height,label,55,xOffsetPosition,yOffsetPosition,AnchorPresets.topLeft);
            if (callback!=null) newButton.button.onClick.AddListener(callback);
            return newButton;
        }

        public UIDynamicSlider AddSlider(JSONStorableFloat jsf, float width, float height, float xOffsetPostion, float yOffsetPostion)
        {
            UIDynamicSlider newSlider = CreateSlider(jsf, width, height, xOffsetPostion, yOffsetPostion);
            newSlider.label = jsf.name;

            return newSlider;
        }

        protected UIDynamicButton InstantiateUIButton()
        {
            Transform buttonTransform = UnityEngine.Object.Instantiate(Globals.mvrScript.manager.configurableButtonPrefab.transform, _spacerActive.gameObject.transform, false);
            UIDynamicButton selectTargetButton = buttonTransform.GetComponent<UIDynamicButton>();
            buttonTransform.SetParent(_spacerActive.gameObject.transform, false);
            return (selectTargetButton);
        }

        protected UIDynamicButton CreateButton(float width, float height, float xOffsetPosition, float yOffsetPostion = 0f)
        {
            RectTransform spacerRT = _spacerActive.GetComponent<RectTransform>();

            spacerRT.anchorMin = new Vector2(0f, 1f);
            spacerRT.anchorMax = new Vector2(0f, 1f);
            spacerRT.pivot = new Vector2(0f, 1f);

            UIDynamicButton targetButton = InstantiateUIButton();

            RectTransform childRect = targetButton.button.GetComponent<RectTransform>();
            childRect.anchorMin = new Vector2(0f, 1f);
            childRect.anchorMax = new Vector2(0f, 1f);
            childRect.pivot = new Vector2(0f,1f);

            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width);
            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
            childRect.transform.localPosition = new Vector3(xOffsetPosition, yOffsetPostion, 0f);

            return (targetButton);
        }

        public void InitUI(int customRowWidth)
        {
            if (customRowWidth != CustomRowWidth.right)
            {
                _spacerLeft = Globals.mvrScript.CreateSpacer(false);
                _spacerActive = _spacerLeft;
                _spacerLeft.height = spacerHeight;


            }
            if (customRowWidth != CustomRowWidth.left)
            {
                _spacerRight = Globals.mvrScript.CreateSpacer(true);
                _spacerRight.height = spacerHeight;
            }
            if (customRowWidth == CustomRowWidth.right)
            {
                _spacerActive = _spacerRight;
            }
        }

    }
    public abstract class CustomRowUIElement : CustomUIElement
    {

        private Dictionary<int, UIDynamicButton> buttonDict = new Dictionary<int, UIDynamicButton>();

        private bool activeButtonHighlight;

        public CustomRowUIElement(float height = 50f, bool _activeButtonHighlight = true) : base(height)
        {
            activeButtonHighlight = _activeButtonHighlight;
        }


        protected void ClearAllButtonColors()
        {
            foreach (KeyValuePair<int, UIDynamicButton> kvp in buttonDict)
            {
                kvp.Value.buttonColor = new Color(0.85f, 0.85f, 0.85f, 1f);
            }
        }

        protected void ClearAllPopupColors(UIDynamicPopup popupException=null)
        {
            foreach (UIDynamicPopup popup in uidPopuplist)
            {
                GameObject popupBackground = popup.popup.transform.Find("Background").gameObject;
                Image backgroundImage = popupBackground.GetComponent<Image>();

                backgroundImage.color = new Color(0.85f, 0.85f, 0.85f, 1f);
                popup.labelTextColor = new Color(0.25f, 0.25f, 0.25f);
                if (popup!= popupException) popup.popup.currentValue = "Select...";
            }
        }
        protected void RegisterButtonAction(UIDynamicButton newButton, string label, int buttonRef, Action<int> callbackMethod)
        {
            newButton.label = label;

            newButton.button.onClick.AddListener(delegate
            {
                try
                {
                    if (activeButtonHighlight) SetButtonColorActive(buttonRef);
                    callbackMethod.Invoke(buttonRef);
                }
                catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
            });

            buttonDict.Add(buttonRef, newButton);
        }

        public void SetButtonColorActive(int buttonRef)
        {
            ClearAllButtonColors();
            ClearAllPopupColors();
            buttonDict[buttonRef].buttonColor = Const.UIhighlightColor;
        }

        public override void ClearUI()
        {
            base.ClearUI();
            buttonDict.Clear();
        }


    }

    public class CustomRowGridlessUIElement : CustomRowUIElement
    {
        public CustomRowGridlessUIElement(float height = 50f, bool _activeButtonHighlight = true) : base(height, _activeButtonHighlight)
        {

        }


        public UIDynamicPopup AddGridlessPopupToRow(JSONStorableStringChooser jssc, UIPopup.OnOpenPopup callbackMethod, float width, float xOffsetPostion, float yOffsetPostion = 0, float panelHeight = 400f, bool scrollable = true)
        {
            UIDynamicPopup newPopup = AddPopup(jssc, callbackMethod, width,spacerHeight, xOffsetPostion, yOffsetPostion, panelHeight);           

            newPopup.popup.onValueChangeHandlers += (string val) =>
            {
                if (val != "Select...")
                {
                    ClearAllButtonColors();
                    ClearAllPopupColors(newPopup);
                    GameObject popupBackground = newPopup.popup.transform.Find("Background").gameObject;
                    Image backgroundImage = popupBackground.GetComponent<Image>();

                    backgroundImage.color = Const.UIhighlightColor;
                    newPopup.labelTextColor = Color.black;
                }
            };       
           
            return newPopup;
        }



        public UIDynamicButton AddGridlessButtonToRow(string label, int buttonRef, Action<int> callbackMethod, float width, float xOffsetPostion, float yOffsetPostion = 0)
        {
            UIDynamicButton newButton = CreateButton(width, spacerHeight, xOffsetPostion, yOffsetPostion);

            RegisterButtonAction(newButton, label, buttonRef, callbackMethod);

            return newButton;
        }

    }

    public class CustomRowGridUIElement : CustomRowUIElement
    {
        private int buttonPadding = 0;


        public CustomRowGridUIElement(float height = 50, bool _activeButtonHighlight = true) : base(height, _activeButtonHighlight)
        {

        }
        public UIDynamicPopup AddPopupToRow(JSONStorableStringChooser jssc, UIPopup.OnOpenPopup callbackMethod = null, float panelHeight = 400f, bool forceRightParentPopup = false, bool scrollable = true)
        {
            UIDynamicPopup newPopup = InstantiatePopup(jssc, scrollable);
            newPopup.label = jssc.label;
            newPopup.popup.onOpenPopupHandlers += callbackMethod;
            newPopup.popup.onOpenPopupHandlers += () =>
            {
                //                UIAPluginUI.CloseAllPopups(newPopup);
                Globals.mvrScript.StartCoroutine(ReparentPopup(newPopup, forceRightParentPopup));
            };
            newPopup.popup.popupPanelHeight = panelHeight;
            newPopup.popup.selectColor = Const.UIhighlightColor;

            uidPopuplist.Add(newPopup);

            return newPopup;
        }

        public UIDynamicButton AddButtonToRow(string label, int buttonRef, Action<int> callbackMethod)
        {
            UIDynamicButton newButton = InstantiateUIButton();
            RegisterButtonAction(newButton, label, buttonRef, callbackMethod);

            return newButton;
        }

        public void InitUI(int customRowWidth, int fixedElementsCount = 0)
        {
            base.InitUI(customRowWidth);

            int width = 525;
            if (customRowWidth == CustomRowWidth.full) width = 1050;

            if (fixedElementsCount > 0)
            {
                GridLayoutGroup glg = _spacerActive.gameObject.AddComponent<GridLayoutGroup>();
                glg.cellSize = new Vector2((width - ((fixedElementsCount - 1) * buttonPadding)) / fixedElementsCount, spacerHeight);

                glg.constraint = GridLayoutGroup.Constraint.FixedRowCount;
                glg.constraintCount = 1;
                glg.padding = new RectOffset(buttonPadding, buttonPadding, buttonPadding, buttonPadding);
                glg.spacing = new Vector2(buttonPadding, 0);
            }


        }

    }
    public static class BAPluginUI
    {
        private static CustomRowGridlessUIElement topTabMenuOptions;

        private static int currentTopTabSelection = TopUITabs.generalSettings;

        private static BaseUITab currentTabUI;

        private static GeneralUITab generalSettingsUITab;
        private static DesktopUITab desktopSettingsUITab;
        private static ResourceSettingsUITab resourceSettingsUITab;

        private static UIDynamicButton generalSettingsUID;
        private static UIDynamicButton VRSettingsUID;
        private static UIDynamicButton desktopSettingsUID;
        private static UIDynamicPopup resourceSettingsUID;

        private static JSONStorableEnumStringChooser resourceSettingsJSESC;

        public static void InitUI()
        {
            topTabMenuOptions = new CustomRowGridlessUIElement(80);

            generalSettingsUITab = new GeneralUITab();
            desktopSettingsUITab = new DesktopUITab();
            resourceSettingsUITab = new ResourceSettingsUITab();
            topTabMenuOptions.InitUI(CustomRowWidth.full);

            resourceSettingsJSESC = new JSONStorableEnumStringChooser("resourceSettings", ResourceType.enumManifestName, ResourceType.scene, "Resource\nSettings", ResourceSettingsTabSelected);

            generalSettingsUID = topTabMenuOptions.AddGridlessButtonToRow("Global\nSettings", TopUITabs.generalSettings, TopTabSelected,200,0,0);
            VRSettingsUID = topTabMenuOptions.AddGridlessButtonToRow("VR\nSettings", TopUITabs.vrSettings, TopTabSelected, 200, 200, 0);
            desktopSettingsUID = topTabMenuOptions.AddGridlessButtonToRow("Desktop\nSettings", TopUITabs.desktopSettings, TopTabSelected, 200, 400,0);
            resourceSettingsUID = topTabMenuOptions.AddGridlessPopupToRow(resourceSettingsJSESC, null, 450,600,0,800,false);
            resourceSettingsUID.popup.currentValue = "Select...";

            generalSettingsUITab.RefreshGroupUI(1);

            topTabMenuOptions.SetButtonColorActive(currentTopTabSelection);

            currentTabUI = generalSettingsUITab;
        }

        private static void ResourceSettingsTabSelected(int val)
        {
            if (currentTopTabSelection!= TopUITabs.resourceSettings)
            {
                currentTopTabSelection = TopUITabs.resourceSettings;
                ClearCurrentTopTabsUI();
                
                currentTabUI = resourceSettingsUITab;

                
            }
            resourceSettingsUITab.resourceSelection = val;

            if (PC.patreonContentEnabled || val == ResourceType.scene || val == ResourceType.presetAppearance) BrowserAssist.browserUI.browserFiltersUI.resourceTypeJSEnum.val = val;
            resourceSettingsUITab.RefreshGroupUI(1);
        }
        private static void TopTabSelected(int tabRef)
        {
            if (tabRef != currentTopTabSelection)
            {
                ClearCurrentTopTabsUI();
                switch (tabRef)
                {
                    case TopUITabs.generalSettings:
                        generalSettingsUITab.RefreshGroupUI(1);
                        currentTabUI = generalSettingsUITab;
                        break;
                    case TopUITabs.vrSettings:
 //                       settingsUI.RefreshGroupUI(LeftRight.both, 1);
                        break;
                    case TopUITabs.desktopSettings:
                        desktopSettingsUITab.RefreshGroupUI(1);
                        currentTabUI = desktopSettingsUITab;
                        break;

                }
                currentTopTabSelection = tabRef;
            }
        }

        private static void ClearCurrentTopTabsUI()
        {
            currentTabUI.ClearAllGroupsUI();

        }

        public static void CloseAllPopups(UIDynamicPopup exception)
        {
            currentTabUI.CloseAllPopups(exception);
        }
    }

    public abstract class BaseUITab
    {
        protected class UIGroup
        {
            public int groupID;
            public BaseUITab uiTab;

            public void RefreshGroupUIStringChange(string val)
            {
                uiTab.RefreshGroupUI(groupID);
            }
        }

        public class UIDynamicElementLists
        {
            public HashSet<UIDynamic> UIDSpacersList;
            public HashSet<UIDynamicButton> UIDButtonsList;
            public HashSet<UIDynamicSlider> UIDSlidersList;
            public HashSet<UIDynamicPopup> UIDPopupList;
            public HashSet<UIDynamicColorPicker> UIDColorPickerList;
            public HashSet<UIDynamicTextField> UIDTextFieldList;
            public HashSet<UIDynamicToggle> UIDToggleList;
            public HashSet<CustomUIElement> UIDCustomElementList;
            public HashSet<BaseUITab> childUITabs;

            public UIDynamicElementLists()
            {
                UIDSpacersList = new HashSet<UIDynamic>();
                UIDButtonsList = new HashSet<UIDynamicButton>();
                UIDSlidersList = new HashSet<UIDynamicSlider>();
                UIDPopupList = new HashSet<UIDynamicPopup>();
                UIDColorPickerList = new HashSet<UIDynamicColorPicker>();
                UIDTextFieldList = new HashSet<UIDynamicTextField>();
                UIDToggleList = new HashSet<UIDynamicToggle>();
                UIDCustomElementList = new HashSet<CustomUIElement>();
                childUITabs = new HashSet<BaseUITab>();
            }

            public void ClearUIDs()
            {
                MVRScript mvrs = Globals.mvrScript;
                foreach (UIDynamic spacer in UIDSpacersList) if (spacer != null) mvrs.RemoveSpacer(spacer);
                foreach (UIDynamicButton button in UIDButtonsList) if (button != null) mvrs.RemoveButton(button);
                foreach (UIDynamicSlider slider in UIDSlidersList) if (slider != null) mvrs.RemoveSlider(slider);
                foreach (UIDynamicPopup popup in UIDPopupList) if (popup != null) mvrs.RemovePopup(popup);
                foreach (UIDynamicColorPicker colorpicker in UIDColorPickerList) if (colorpicker != null) mvrs.RemoveColorPicker(colorpicker);
                foreach (UIDynamicTextField textfield in UIDTextFieldList) if (textfield != null) mvrs.RemoveTextField(textfield);
                foreach (UIDynamicToggle toggle in UIDToggleList) if (toggle != null) mvrs.RemoveToggle(toggle);
                foreach (CustomUIElement Element in UIDCustomElementList) Element.ClearUI();

                foreach (BaseUITab childUITab in childUITabs) childUITab.ClearAllGroupsUI();
                UIDSpacersList.Clear();
                UIDButtonsList.Clear();
                UIDSlidersList.Clear();
                UIDPopupList.Clear();
                UIDColorPickerList.Clear();
                UIDTextFieldList.Clear();
                UIDToggleList.Clear();
                UIDCustomElementList.Clear();
            }

            public int GetUIElementsCount()
            {
                int count = 0;
                foreach (UIDynamic spacer in UIDSpacersList) count += 1;
                foreach (UIDynamicButton button in UIDButtonsList) count += 1;
                foreach (UIDynamicSlider slider in UIDSlidersList) count += 1;
                foreach (UIDynamicPopup popup in UIDPopupList) count += 1;
                foreach (UIDynamicColorPicker colorpicker in UIDColorPickerList) count += 1;
                foreach (UIDynamicTextField textfield in UIDTextFieldList) count += 1;
                foreach (UIDynamicToggle toggle in UIDToggleList) count += 1;
                foreach (CustomUIElement customElement in UIDCustomElementList) count += 1;
                foreach (BaseUITab childUITab in childUITabs) count += 1;

                return count;
            }
            public void CloseAllPopup(UIDynamicPopup exception)
            {
                foreach (UIDynamicPopup UIDpopup in UIDPopupList)
                {
                    if (UIDpopup != exception) UIDpopup.popup.visible = false;
                }
                foreach (CustomUIElement customElement in UIDCustomElementList)
                {
                    customElement.CloseAllPopups(exception);
                }
                foreach (BaseUITab childUITab in childUITabs)
                {
                    childUITab.CloseAllPopups(exception);
                }
            }
        }

        private Dictionary<int, UIDynamicElementLists> uiElementGroups;
        private Dictionary<int, List<JSONStorableParam>> JSPsGroups;

        protected int uiGroupCount = 1;

        public BaseUITab()
        {
            uiElementGroups = new Dictionary<int, UIDynamicElementLists>();
            JSPsGroups = new Dictionary<int, List<JSONStorableParam>>();
        }
        protected void ForceUIRefreshOnValChange(JSONStorableParam jsp, int uiRefreshGroup, BaseUITab uiTab = null)
        {
            UIGroup groupLR = new UIGroup();
            groupLR.groupID = uiRefreshGroup;

            if (uiTab == null) groupLR.uiTab = this;
            else groupLR.uiTab = uiTab;

            switch (jsp.type)
            {
                case JSONStorable.Type.StringChooser:
                    RegisterJSSCRefreshUIonValChange(jsp as JSONStorableStringChooser, groupLR);
                    break;
                case JSONStorable.Type.String:
                    RegisterJSSRefreshUIonValChange(jsp as JSONStorableString, groupLR);
                    break;
                case JSONStorable.Type.Float:
                    RegisterJSFRefreshUIonValChange(jsp as JSONStorableFloat, groupLR);
                    break;
                case JSONStorable.Type.Bool:
                    RegisterJSBRefreshUIonValChange(jsp as JSONStorableBool, groupLR);
                    break;
                default:
                    SuperController.LogMessage("BA.BaseUITab.RegisterJSRefreshUIonValChange: unsupported JSONStorableParam type " + jsp.type.ToString());
                    break;
            }
        }
        private void RegisterJSSCRefreshUIonValChange(JSONStorableStringChooser jssc, UIGroup groupLR)
        {

            if (jssc.popup == null)
            {
                SuperController.LogMessage("BA.BaseUITab.RegisterJSSCRefreshUIonValChange - attempt to register JSSC with no associated UI Popup");
                return;
            }
            jssc.popup.onValueChangeHandlers += groupLR.RefreshGroupUIStringChange;
        }

        private void RegisterJSBRefreshUIonValChange(JSONStorableBool jsb, UIGroup groupLR)
        {
            if (jsb.toggle == null)
            {
                SuperController.LogMessage("BA.BaseUITab.RegisterJSBRefreshUIonValChange - attempt to register JSB with no associated UI Toggle");
                return;
            }
            jsb.toggle.onValueChanged.AddListener((value) => groupLR.RefreshGroupUIStringChange(""));
        }

        private void RegisterJSSRefreshUIonValChange(JSONStorableString jss, UIGroup groupLR)
        {
            if (jss.inputField == null)
            {
                SuperController.LogMessage("BA.BaseUITab.RegisterJSSRefreshUIonValChange - attempt to register JSS with no associated inputfield");
                return;
            }
            jss.inputField.onValueChanged.AddListener((text) => groupLR.RefreshGroupUIStringChange(text));

        }

        private void RegisterJSFRefreshUIonValChange(JSONStorableFloat jsf, UIGroup groupLR)
        {

            if (jsf.slider == null)
            {
                SuperController.LogMessage("BA.BaseUITab.RegisterJSFRefreshUIonValChange - attempt to register JSF with no associated Slider");
                return;
            }
            jsf.slider.onValueChanged.AddListener((floatVal) => groupLR.RefreshGroupUIStringChange(floatVal.ToString()));

        }

        protected abstract void CreateGroupUI(int groupID);

        public void RefreshGroupUI(int startGroupID)
        {

            CloseAllPopups(null);

            for (int groupID = startGroupID; groupID <= uiGroupCount; groupID++)
            {
                ClearGroupUI(groupID);
            }

            for (int groupID = startGroupID; groupID <= uiGroupCount; groupID++)
            {
                CreateGroupUI(groupID);
            }
        }

        private UIDynamicElementLists GetUIElementsFromGroup(int groupID, Dictionary<int, UIDynamicElementLists> uiElementsGroups)
        {
            UIDynamicElementLists uiElementGroup;

            if (uiElementsGroups.ContainsKey(groupID)) uiElementGroup = uiElementsGroups[groupID];
            else
            {
                uiElementGroup = new UIDynamicElementLists();
                uiElementsGroups.Add(groupID, uiElementGroup);
            }
            return (uiElementGroup);
        }

        protected void RegisterCustomUIElement(int groupID, CustomUIElement customElement)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);     

            uiElements.UIDCustomElementList.Add(customElement);
        }

        protected void RegisterChildUITab(int groupID, BaseUITab childUITab)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);
            uiElements.childUITabs.Add(childUITab);
        }

        protected UIDynamic CreateSpacer(int groupID, float height = 10f, bool rightSide = false)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);

            UIDynamic spacer = Globals.mvrScript.CreateSpacer(rightSide);
            uiElements.UIDSpacersList.Add(spacer);
            spacer.height = height;
            return (spacer);
        }

        public void ClearAllGroupsUI()
        {
            CloseAllPopups(null);

            foreach (KeyValuePair<int, UIDynamicElementLists> kvp in uiElementGroups) ClearGroupUI(kvp.Key);

        }
        public virtual void ClearGroupUI(int groupID)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);

            List<JSONStorableParam> jspListForGroup = null;
            uiElements.ClearUIDs();
            if (JSPsGroups.ContainsKey(groupID)) jspListForGroup = JSPsGroups[groupID];

            if (jspListForGroup != null)
            {
                jspListForGroup.Clear();
            }

        }
        public void CloseAllPopups(UIDynamicPopup exception)
        {
            foreach (var uiElements in uiElementGroups.Values.ToList()) uiElements.CloseAllPopup(exception);
        }

    }

    public class GeneralUITab : BaseUITab
    {
        private CustomUIElement generalSettingsCustomUI;
        public GeneralUITab()
        {
            generalSettingsCustomUI = new CustomUIElement(600);
        }

        protected override void CreateGroupUI(int groupID)
        {
            generalSettingsCustomUI.InitUI(CustomRowWidth.full);

            generalSettingsCustomUI.AddToggle(Settings.enableTopScreenLaunchButtonJSB, 1020, 50, 0, 0, "Enable BrowserAssist Launch button on VAM Top Menu");
            generalSettingsCustomUI.AddPopup(Settings.launchButtonPlacementJSESC, null, 1020, 80, 0, -60, labelProportion: 0.7f);
            generalSettingsCustomUI.AddPopup(Settings.defaultResourceTypeJESC, null, 1020, 80, 0, -150, labelProportion: 0.7f);
            generalSettingsCustomUI.AddPopup(Settings.resourceNameSearchModeJSESC, null, 1020, 80, 0, -240, labelProportion: 0.7f);

            var maxButtonsSlider = generalSettingsCustomUI.AddSlider(Settings.maxButtonsPerPageJSF, 1020, 120, 0, -330);
            maxButtonsSlider.valueFormat = "F0";
            maxButtonsSlider.ConfigureQuickButtons(-10f, -50f, -100f, -1000f, 10f, 50f, 100f, 1000f);

            RegisterCustomUIElement(groupID, generalSettingsCustomUI);
        }
    }

    public class DesktopUITab : BaseUITab
    {
        private CustomUIElement desktopSettingsCustomUI;
        public DesktopUITab()
        {
            desktopSettingsCustomUI = new CustomUIElement(600);
        }

        protected override void CreateGroupUI(int groupID)
        {
            desktopSettingsCustomUI.InitUI(CustomRowWidth.full);

            var uiScaleSlider = desktopSettingsCustomUI.AddSlider(Settings.uiScaleJSF, 1020, 120, 0, 0);
            uiScaleSlider.valueFormat = "F2";
            uiScaleSlider.ConfigureQuickButtons(-0.1f, -0.25f, -0.5f, -1f, 0.1f, 0.25f, 0.5f, 1f);

            RegisterCustomUIElement(groupID, desktopSettingsCustomUI);
        }
    }

    public class ResourceSettingsUITab : BaseUITab
    {
        private CustomRowGridUIElement resourceSettingsTabMenuOptions;
        public int resourceSelection = ResourceType.scene;
        private CustomUIElement resourceSettingsCustomUI;

        private ResourceSettings resourceSettings { get { return Settings.resourceSettings[resourceSelection]; } }

        private int currentResourceSelectionSettingsTab = ResourceSettingsUITabs.defaultFilters ;

        public ResourceSettingsUITab()
        {
            resourceSettingsTabMenuOptions = new CustomRowGridUIElement();
            uiGroupCount = 2;

            resourceSettingsCustomUI = new CustomUIElement(600);
        }

        protected override void CreateGroupUI(int groupID)
        {
            if (groupID == 1)
            {
                resourceSettingsTabMenuOptions.InitUI(CustomRowWidth.full, 4);
                resourceSettingsTabMenuOptions.AddButtonToRow("Default Filters", ResourceSettingsUITabs.defaultFilters, ResourceSettingsTabSelected);
                RegisterCustomUIElement(groupID, resourceSettingsTabMenuOptions);

                resourceSettingsTabMenuOptions.SetButtonColorActive(currentResourceSelectionSettingsTab);
            }
            if (groupID == 2)
            {
                var resourceTypeDisplayName = EnumManifest.GetDisplayVal(ResourceType.enumManifestName, resourceSettings.defaultFilterSettings.resourceType);
                resourceSettingsCustomUI.InitUI(CustomRowWidth.full);
                RegisterCustomUIElement(groupID, resourceSettingsCustomUI);
                if (currentResourceSelectionSettingsTab == ResourceSettingsUITabs.defaultFilters)
                {
                    if (PC.patreonContentEnabled || resourceSelection == ResourceType.scene || resourceSelection == ResourceType.presetAppearance)
                    {
                        resourceSettingsCustomUI.AddPopup(resourceSettings.defaultFilterSettings.selectModeJSEnum, null, 520, 50, 0, 0, labelProportion: 0.6f, labelOverride: "Default Select Mode");

                        var tagPopupUID = resourceSettingsCustomUI.AddPopup(resourceSettings.defaultFilterSettings.tagsFilterJSMultiEnum, null, 520, 50, 530, 0, labelProportion: 0.6f, labelOverride: "Default Tag Filter", panelHeight: 600f);
                        tagPopupUID.popup.onOpenPopupHandlers += resourceSettings.defaultFilterSettings.RefreshTagFilters;

                        resourceSettingsCustomUI.AddToggle(resourceSettings.defaultFilterSettings.showHiddenJSB, 300, 50, 0, -60, "Show Hidden");
                        resourceSettingsCustomUI.AddToggle(resourceSettings.defaultFilterSettings.onlyFavouritesJSB, 300, 50, 310, -60, "Only Favorites");
                        resourceSettingsCustomUI.AddToggle(resourceSettings.defaultFilterSettings.onlyNewJSB, 300, 50, 620, -60, "Only New");

                        var creatorPopupUID = resourceSettingsCustomUI.AddPopup(resourceSettings.defaultFilterSettings.creatorJSMultiEnum, null, 720, 100, 0, -120, labelProportion: 0.6f, labelOverride: "Default Creator / Local filter: Packages or Local content", panelHeight: 800f);
                        creatorPopupUID.popup.onOpenPopupHandlers += resourceSettings.defaultFilterSettings.RefreshCreatorFilters;
                        ForceUIRefreshOnValChange(resourceSettings.defaultFilterSettings.creatorJSMultiEnum, 2);

                        if (resourceSettings.defaultFilterSettings.creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag || resourceSettings.defaultFilterSettings.creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.allVARs)
                        {
                            resourceSettingsCustomUI.AddPopup(resourceSettings.defaultFilterSettings.licenseFilterJSEnum, null, 320, 100, 730, -120, labelProportion: 0.6f, labelOverride: "Default VAR License");
                        }
                        resourceSettingsCustomUI.AddPopup(resourceSettings.defaultFilterSettings.sortModeJSEnum, null, 400, 50, 0, -230);

                        
                        var resetButton = resourceSettingsCustomUI.AddButton(450f, 120f, 300f, -500f, "Set '" + resourceTypeDisplayName + "' default Filter values to the current '" + resourceTypeDisplayName + "' Filter settings", ResetFilterSettings);
                        resetButton.buttonColor = Color.yellow;
                    }
                    else
                    {
                        resourceSettingsCustomUI.AddTextBox("JayJayWon Patreon subscribers have access to the full BrowserAssist plugin which allows for browsing of additional resource types, including "+ resourceTypeDisplayName, 1000, 200, 0, 10);
                    }
                        
                }
                
            }
            


        }
        private void ResetFilterSettings()
        {
            var currentBrowserResourceFilterSettings = BrowserAssist.browserUI.resourceBrowserUI.browserFiltersUI.filterSettingsPerResourceType[resourceSelection];

            resourceSettings.defaultFilterSettings.RefreshTagFilters();

            resourceSettings.defaultFilterSettings.selectModeJSEnum.val = currentBrowserResourceFilterSettings.selectModeJSEnum.val;
            resourceSettings.defaultFilterSettings.tagsFilterJSMultiEnum.val = currentBrowserResourceFilterSettings.tagsFilterJSMultiEnum.val;
            resourceSettings.defaultFilterSettings.showHiddenJSB.valNoCallback = currentBrowserResourceFilterSettings.showHiddenJSB.val;
            resourceSettings.defaultFilterSettings.onlyFavouritesJSB.valNoCallback = currentBrowserResourceFilterSettings.onlyFavouritesJSB.val;
            resourceSettings.defaultFilterSettings.onlyNewJSB.valNoCallback = currentBrowserResourceFilterSettings.onlyNewJSB.val;
            resourceSettings.defaultFilterSettings.RefreshCreatorFilters();
            resourceSettings.defaultFilterSettings.creatorJSMultiEnum.val = currentBrowserResourceFilterSettings.creatorJSMultiEnum.val;
            resourceSettings.defaultFilterSettings.RefreshLicenseFilters();
            resourceSettings.defaultFilterSettings.licenseFilterJSEnum.valNoCallback = currentBrowserResourceFilterSettings.licenseFilterJSEnum.val;
            resourceSettings.defaultFilterSettings.sortModeJSEnum.valNoCallback = currentBrowserResourceFilterSettings.sortModeJSEnum.val;
            Settings.SaveSettings();
        }

        private void ResourceSettingsTabSelected(int tabRef)
        {
            if (tabRef != currentResourceSelectionSettingsTab)
            {
                RefreshGroupUI(2);

                currentResourceSelectionSettingsTab = tabRef;
            }
        }

    }
}
