﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    public class Settings
    {

        public static JSONStorableFloat maxButtonsPerPageJSF = new JSONStorableFloat("Maximum Number of Resources per Browser page",200f,MaxButtonsPerPageCallback, 20f,2000f);
        public static JSONStorableFloat uiScaleJSF = new JSONStorableFloat("Desktop UI Scale", 1f, UIScaleCallback, 0.5f, 2f);
        public static JSONStorableEnumStringChooser resourceNameSearchModeJSESC = new JSONStorableEnumStringChooser("resourceNameSearchMode", ResourceNameSearchMode.enumManifestName,ResourceNameSearchMode.ContainsFullPath,"Resource Filter Name Search Mode", ResourceNameSearchModeCallback);
        public static JSONStorableEnumStringChooser launchButtonPlacementJSESC = new JSONStorableEnumStringChooser("launchButtonPlacement", VAMUILaunchButtonPosition.enumManifestName, VAMUILaunchButtonPosition.sceneBrowserSub, "BrowserAssist Launch Button Position on VAM UI", LaunchButtonPlacementCallback);

        private static int _resourceButtonSizeVR=4;
        private static int _resourceButtonSizeDesktop=4;
        public static int resourceButtonSizeVR { get{ return _resourceButtonSizeVR; } set { _resourceButtonSizeVR = value;SaveSettings(); } }
        public static int resourceButtonSizeDesktop { get { return _resourceButtonSizeDesktop; } set { _resourceButtonSizeDesktop = value; SaveSettings(); } }

        public static JSONStorableBool enableTopScreenLaunchButtonJSB = new JSONStorableBool("EnableTopScreenLaunchButton", true, EnableTopScreenLaunchButtonChange);
        public static JSONStorableEnumStringChooser defaultResourceTypeJESC = new JSONStorableEnumStringChooser("defaultResourceType", ResourceType.enumManifestName, ResourceType.scene, "Initial Resource Type on BrowserAssist startup",SettingsStringChange);

        public static Dictionary<int, ResourceSettings> resourceSettings = new Dictionary<int, ResourceSettings>();

        private static bool restoreInProgress = false;

        public Settings()
        {
            
        }

        private static void EnableTopScreenLaunchButtonChange(bool i)
        {
            SaveSettings();
            if (!restoreInProgress) Globals.mvrScript.ToggleTopMenuLanchButton();
        }

        
        private static void LaunchButtonPlacementCallback(string val)
        {
            SaveSettings();
            if (!restoreInProgress) Globals.mvrScript.RePositionLaunchButton();
        }
        private static void MaxButtonsPerPageCallback(float val)
        {
            maxButtonsPerPageJSF.valNoCallback = Mathf.Round(val / 10f) *10f;
            if (BrowserAssist.browserUI!=null && BrowserAssist.browserUI.resourceBrowserUI != null && BrowserAssist.browserUI.isUIActive) BrowserAssist.browserUI.RefreshButtonLayout();
            SaveSettings();
        }

        private static void ResourceNameSearchModeCallback(string val)
        {
            if (BrowserAssist.browserUI != null) BrowserAssist.browserUI.browserFiltersUI.SetResourceNameFilterHelpText();
            SaveSettings();
        }
        private static void UIScaleCallback(float val)
        {
            if (BrowserAssist.browserUI != null) BrowserAssist.browserUI.RefreshScale();
            SaveSettings();
        }
        private static void SettingsBoolChange(bool val)
        {
            SaveSettings();
        }

        private static void SettingsFloatChange(float val)
        {
            SaveSettings();
        }

        private static void SettingsStringChange(string val)
        {
            SaveSettings();
        }

        public static void SaveSettings()
        {
            if (!restoreInProgress)
            {
                JSONClass settingsJC = new JSONClass();

                maxButtonsPerPageJSF.StoreJSON(settingsJC["BrowserSettings"].AsObject);
                uiScaleJSF.StoreJSON(settingsJC["DesktopSettings"].AsObject);
                resourceNameSearchModeJSESC.StoreJSON(settingsJC["FilterSettings"].AsObject);
                launchButtonPlacementJSESC.StoreJSON(settingsJC["GeneralSettings"].AsObject);
                settingsJC["BrowserSettings"]["ResourceButtonSizeVR"].AsInt = _resourceButtonSizeVR;
                settingsJC["BrowserSettings"]["ResourceButtonSizeDesktop"].AsInt = _resourceButtonSizeDesktop;
                defaultResourceTypeJESC.StoreJSON(settingsJC["BrowserSettings"].AsObject);
                enableTopScreenLaunchButtonJSB.StoreJSON(settingsJC["GeneralSettings"].AsObject);

                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    string resourceTypeStoreLabel = EnumManifest.GetStoreVal(ResourceType.enumManifestName, resourceType);

                    settingsJC["ResourceSettings"][resourceTypeStoreLabel] = resourceSettings[resourceType].GetJSON();
                }

                SuperController.singleton.SaveJSON(settingsJC, Const.BAScriptDataFolder + "\\" + "BASettings.cfg");
            }

        }

        public static void LoadSettings()
        {
            List<int> resourceTypesToExclude = new List<int>();
            if (!PC.patreonContentEnabled)
            {
                resourceTypesToExclude = EnumManifest.GetEnumsList(ResourceType.enumManifestName);
                resourceTypesToExclude.Remove(ResourceType.scene);
                resourceTypesToExclude.Remove(ResourceType.presetAppearance);
            }

            defaultResourceTypeJESC.SetEnumChoices(ResourceType.enumManifestName, resourceTypesToExclude);

            if (FileManagerSecure.FileExists(Const.BAScriptDataFolder + "\\" + "BASettings.cfg"))
            {
                
                JSONClass settingsJC;
                try
                {
                    settingsJC = SuperController.singleton.LoadJSON(Const.BAScriptDataFolder + "\\" + "BASettings.cfg").AsObject;
                }
                catch
                {
                    Utils.LogDebug("Settings.LoadSettings: Error during load of settings config file. Possible malformed JSON");
                    return;
                }

                restoreInProgress = true;

                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    ResourceSettings newResourceSettings = new ResourceSettings(resourceType);
                    
                    string resourceTypeStoreLabel = EnumManifest.GetStoreVal(ResourceType.enumManifestName, resourceType);

                    newResourceSettings.RestoreJSON(settingsJC["ResourceSettings"][resourceTypeStoreLabel].AsObject);

                    resourceSettings.Add(resourceType, newResourceSettings);
                }

                uiScaleJSF.RestoreFromJSON(settingsJC["DesktopSettings"].AsObject);
                maxButtonsPerPageJSF.RestoreFromJSON(settingsJC["BrowserSettings"].AsObject);
                resourceNameSearchModeJSESC.RestoreFromJSON(settingsJC["FilterSettings"].AsObject);
                launchButtonPlacementJSESC.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);
                if (settingsJC["BrowserSettings"]["ResourceButtonSizeVR"]!=null) _resourceButtonSizeVR = settingsJC["BrowserSettings"]["ResourceButtonSizeVR"].AsInt;
                if (settingsJC["BrowserSettings"]["ResourceButtonSizeDesktop"] != null) _resourceButtonSizeDesktop = settingsJC["BrowserSettings"]["ResourceButtonSizeDesktop"].AsInt;
                
                defaultResourceTypeJESC.RestoreFromJSON(settingsJC["BrowserSettings"].AsObject);
                enableTopScreenLaunchButtonJSB.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);

                restoreInProgress = false;
            }
            else
            {
                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    ResourceSettings newResourceSettings = new ResourceSettings(resourceType);
                    resourceSettings.Add(resourceType, newResourceSettings);
                }
                SaveSettings();
            }

        }
    }

    public class ResourceSettings
    {
        public JSONStorableBool closeBrowserUIonResourceLoadJSB = new JSONStorableBool("closeBrowserUIonResourceLoad", false, SettingsBoolChange);
        public ResourceBrowserFilterSettings defaultFilterSettings;
        public ResourceSettings(int resourceType)
        {
            defaultFilterSettings = new ResourceBrowserFilterSettings(resourceType);

        }

        public virtual JSONClass GetJSON()
        {
            JSONClass resourceSettingsJC = new JSONClass();
            closeBrowserUIonResourceLoadJSB.StoreJSON(resourceSettingsJC);
            resourceSettingsJC["defaultFilterSettings"] = defaultFilterSettings.GetJSON();

            return resourceSettingsJC;
        }

        public virtual void RestoreJSON(JSONClass resourceSettingsJC)
        {
            closeBrowserUIonResourceLoadJSB.RestoreFromJSON(resourceSettingsJC);

            defaultFilterSettings.RestoreJSON(resourceSettingsJC["defaultFilterSettings"].AsObject);
        }

        private static void SettingsBoolChange(bool val)
        {
            Settings.SaveSettings();
        }
    }

    public class ResourceBrowserFilterSettings
    {
        public JSONStorableMultiEnumStringChooser creatorJSMultiEnum;
        public JSONStorableStringChooser folderFilterJSSC;
        public JSONStorableString resourceNameFilterJSS;

        public JSONStorableBool showHiddenJSB;
        public JSONStorableBool onlyFavouritesJSB;
        public JSONStorableBool onlyNewJSB;

        public JSONStorableMultiEnumStringChooser tagsFilterJSMultiEnum;
        public JSONStorableEnumStringChooser licenseFilterJSEnum;

        public JSONStorableEnumStringChooser selectModeJSEnum;
        public JSONStorableEnumStringChooser sortModeJSEnum;

        public string resourceNameFilterQuick;

        public int resourceType { get; private set; }

        public ResourceBrowserFilterSettings(int _resourceType)
        {
            resourceType = _resourceType;

            var tagNames = TagManifest.GetTagNames(resourceType);
            tagsFilterJSMultiEnum = new JSONStorableMultiEnumStringChooser("tagsFilter", tagNames, tagNames, null, null, SettingsTagMainValChange, TagGroups.enumManifestName, SettingsTagTopValChange);

            

            showHiddenJSB = new JSONStorableBool("showHidden", false, SettingsBoolChange);
            onlyFavouritesJSB = new JSONStorableBool("onlyFavourites", false, SettingsBoolChange);
            onlyNewJSB = new JSONStorableBool("onlyNew", false, SettingsBoolChange);

            List<string> creators = CreatorManifest.GetAllCreatorNamesList();
            creatorJSMultiEnum = new JSONStorableMultiEnumStringChooser("creatorFilter", creators, creators, null, null, SettingsCreatorMainValChange, CreatorFilterChoices.enumManifestName, SettingsCreatorTopValChange);
            creatorJSMultiEnum.valTopEnumNoCallback = CreatorFilterChoices.allVARs;

            folderFilterJSSC = new JSONStorableStringChooser("folderFilter", null, "", "Folder");
            resourceNameFilterJSS = new JSONStorableString("resourceNameFilter", "");
            resourceNameFilterQuick = "";

            licenseFilterJSEnum = new JSONStorableEnumStringChooser("licenseFilter", LicenseType.enumManifestName, LicenseType.Any, "Package License", SettingsIntChange);

            selectModeJSEnum = new JSONStorableEnumStringChooser("selectMode", BrowserSelectMode.enumManifestName, BrowserSelectMode.single, "Select Mode", SettingsIntChange);
            sortModeJSEnum = new JSONStorableEnumStringChooser("sortMode", BrowserSortMode.enumManifestName, BrowserSortMode.cataloguedDateTime, "Order by", SettingsIntChange);
            
        }
        public ResourceBrowserFilterSettings(BrowserFiltersUI parentFiltersUI, int _resourceType)
        {
            resourceType = _resourceType;

            var tagNames = TagManifest.GetTagNames(resourceType);
            tagsFilterJSMultiEnum = new JSONStorableMultiEnumStringChooser("tagsFilter", tagNames, tagNames, null, null, parentFiltersUI.TagMainValCallback, TagGroups.enumManifestName, parentFiltersUI.TagTopValCallback);
            tagsFilterJSMultiEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.tagsFilterJSMultiEnum.val;
            tagsFilterJSMultiEnum.label = "Tags";          

            showHiddenJSB = new JSONStorableBool("showHidden", Settings.resourceSettings[resourceType].defaultFilterSettings.showHiddenJSB.val, parentFiltersUI.ShowHiddenFilterCallback);
            onlyFavouritesJSB = new JSONStorableBool("onlyFavourites", Settings.resourceSettings[resourceType].defaultFilterSettings.onlyFavouritesJSB.val, parentFiltersUI.OnlyFavFilterCallback);
            onlyNewJSB = new JSONStorableBool("onlyNew", Settings.resourceSettings[resourceType].defaultFilterSettings.onlyNewJSB.val, parentFiltersUI.OnlyNewFilterCallback);

            List<string> creators = CreatorManifest.GetAllCreatorNamesList();
            creatorJSMultiEnum = new JSONStorableMultiEnumStringChooser("creatorFilter", creators, creators, null, null, parentFiltersUI.CreatorsMainValCallback, CreatorFilterChoices.enumManifestName, parentFiltersUI.CreatorsTopValCallback);
            creatorJSMultiEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val;
            creatorJSMultiEnum.label = "Creator / Local";

            folderFilterJSSC = new JSONStorableStringChooser("folderFilter", null, "", "Folder");
            resourceNameFilterJSS = new JSONStorableString("resourceNameFilter", "");
            resourceNameFilterQuick = "";

            licenseFilterJSEnum = new JSONStorableEnumStringChooser("licenseFilter", LicenseType.enumManifestName, Settings.resourceSettings[resourceType].defaultFilterSettings.licenseFilterJSEnum.val, "Package License", parentFiltersUI.LicenseCallback);

            selectModeJSEnum = new JSONStorableEnumStringChooser("selectMode", BrowserSelectMode.enumManifestName, Settings.resourceSettings[resourceType].defaultFilterSettings.selectModeJSEnum.val, "Select Mode", parentFiltersUI.SelectModeCallback);
            sortModeJSEnum = new JSONStorableEnumStringChooser("sortMode", BrowserSortMode.enumManifestName, Settings.resourceSettings[resourceType].defaultFilterSettings.sortModeJSEnum.val, "Order by", parentFiltersUI.SortByCallback);

            
        }
        private void SettingsTagMainValChange(string i)
        {
            RefreshCreatorFilters();
            Settings.SaveSettings();
        }

        private void SettingsTagTopValChange(int i)
        {
            RefreshCreatorFilters();
            Settings.SaveSettings();
        }
        private void SettingsCreatorMainValChange(string i)
        {
            RefreshLicenseFilters();
            Settings.SaveSettings();
        }

        private void SettingsCreatorTopValChange(int i)
        {
            RefreshLicenseFilters();
            Settings.SaveSettings();
        }

        public virtual JSONClass GetJSON()
        {
            JSONClass defaultFilterSettingsJC = new JSONClass();
            creatorJSMultiEnum.StoreJSON(defaultFilterSettingsJC);

            showHiddenJSB.StoreJSON(defaultFilterSettingsJC);
            onlyFavouritesJSB.StoreJSON(defaultFilterSettingsJC);
            onlyNewJSB.StoreJSON(defaultFilterSettingsJC);

            tagsFilterJSMultiEnum.StoreJSON(defaultFilterSettingsJC);
            licenseFilterJSEnum.StoreJSON(defaultFilterSettingsJC);

            selectModeJSEnum.StoreJSON(defaultFilterSettingsJC);
            sortModeJSEnum.StoreJSON(defaultFilterSettingsJC);

            return defaultFilterSettingsJC;
        }

        public virtual void RestoreJSON(JSONClass defaultFilterSettingsJC)
        {
            creatorJSMultiEnum.RestoreFromJSON(defaultFilterSettingsJC);

            showHiddenJSB.RestoreFromJSON(defaultFilterSettingsJC);
            onlyFavouritesJSB.RestoreFromJSON(defaultFilterSettingsJC);
            onlyNewJSB.RestoreFromJSON(defaultFilterSettingsJC);

            tagsFilterJSMultiEnum.RestoreFromJSON(defaultFilterSettingsJC);
            licenseFilterJSEnum.RestoreFromJSON(defaultFilterSettingsJC);

            selectModeJSEnum.RestoreFromJSON(defaultFilterSettingsJC);
            sortModeJSEnum.RestoreFromJSON(defaultFilterSettingsJC);
        }

        private static void SettingsBoolChange(bool val)
        {
            Settings.SaveSettings();
        }
        private static void SettingsStringChange(string val)
        {
            Settings.SaveSettings();
        }

        private static void SettingsIntChange(int val)
        {
            Settings.SaveSettings();
        }

        public void RefreshTagFilters()
        {
            List<string> tags = TagManifest.GetTagNames(resourceType);
            tagsFilterJSMultiEnum.SetMainChoices(tags);
            if (tagsFilterJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag)
            {
                if (!tags.Contains(tagsFilterJSMultiEnum.mainVal)) tagsFilterJSMultiEnum.valTopEnum = TagGroups.anyTags;
            }

            List<int> ignoreTagGroups = new List<int>();
            var tagCategories = TagManifest.GetTagCategories(resourceType).ToList();
            tagCategories.Remove(TagCategory.userDefined);
            if (tagCategories.Count == 0) ignoreTagGroups.Add(TagGroups.noUserTags);

            tagsFilterJSMultiEnum.SetTopEnum(TagGroups.enumManifestName, ignoreTagGroups);

            if (tagsFilterJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.topEnumValFlag)
            {
                if (ignoreTagGroups.Contains(tagsFilterJSMultiEnum.valTopEnum)) tagsFilterJSMultiEnum.valTopEnum = TagGroups.anyTags;
            }
        }

        public void RefreshCreatorFilters()
        {
            List<string> creators = CreatorManifest.GetFilteredCreatorNames(this);
            List<string> displayCreators = CreatorManifest.GetFilteredCreatorNamesWithResourceCount(this);

            creatorJSMultiEnum.SetMainChoices(creators, displayCreators, false);

            if (creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag)
            {
                if (!creators.Contains(creatorJSMultiEnum.mainVal)) creatorJSMultiEnum.valTopEnum = CreatorFilterChoices.allVARs;
            }
        }

        public void RefreshLicenseFilters()
        {
            if (creatorJSMultiEnum.valType!=JSONStorableMultiEnumStringChooser.topEnumValFlag || creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.allVARs)
            {
                licenseFilterJSEnum.SetEnumChoices(LicenseType.enumManifestName, ResourceManifest.GetLicenseTypesByInverseFilters(resourceType, showHiddenJSB.val, onlyFavouritesJSB.val, onlyNewJSB.val, tagsFilterJSMultiEnum, creatorJSMultiEnum));
            }
        }
    }
}
